# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse

from .meta import NAME, DESCRIPTION
from . import __version__
parser = argparse.ArgumentParser(
    prog='{} {}'.format(NAME, __version__),
    description=DESCRIPTION,
    epilog='The Cake is a lie.',
)

group_main = parser.add_argument_group(title='Main options')

group_main.add_argument(
    '-i', '--input',
    help='Input files to configuration profile set',
    default=[],
    action='append'
)

group_main.add_argument(
    '-o', '--output',
    help='Output path to generated files to'
)

# Group Logging
group_logging = parser.add_argument_group(title='Logging options')

group_logging.add_argument(
    '-q', '--quiet',
    help='Keep output to minimum, only requested data (listing) or errors',
    action='store_true'
)

group_logging.add_argument(
    '-v', '--verbose',
    help='Print generation status and user relevant info',
    action='store_true'
)

group_logging.add_argument(
    '-d', '--debug',
    help='Print debugging details',
    action='store_true'
)

# Group Misc
group_misc = parser.add_argument_group(title='Miscellaneous')

group_misc.add_argument(
    '--version',
    help='Display version information',
    action='store_true'
)



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import re

from .exceptions import ProfileError, TemplateError
from .meta import PROFILES, TEMPLATES

LOG = logging.getLogger(__name__)

REX_TEMPLATE_TO_OUTPUT = re.compile(r'^(.*)(\.jinja2)$')


def get_module_path():
    """Get module installation path,
    for reading packaged template sets and profiles

    :return: module installation path
    :rtype: str
    """
    module_path = os.path.dirname(__file__)
    LOG.debug('Module path: %s', module_path)
    return module_path


def get_profiles_path():
    """Helper for getting profiles path from module installation location

    :return: path to packaged profiles directory
    :rtype: str
    """
    module_path = get_module_path()
    profiles_path = os.path.join(module_path, PROFILES)
    return profiles_path


def get_templates_path():
    """Helper for getting templates path from module installation location

    :return: path packaged templates directory
    :rtype: str
    """
    module_path = get_module_path()
    templates_path = os.path.join(module_path, TEMPLATES)
    return templates_path


def select_profile_file(profile_name):
    """Select profile path and filename (user defined or from package)

    :param profile_name: profile name, from package or filename from user
    :type profile_name: str

    :raises ProfileError: when requested profile file does not exists

    :return: selected path to profile and profile name
    :rtype: tuple[str, str]
    """
    profiles_path = get_profiles_path()
    selected_template_path = profiles_path
    selected_template_name = profile_name

    user_extra_path = os.path.join(PROFILES, profile_name)
    if os.path.isfile(user_extra_path):  # user path omitting 'profile' dir
        profile_tmp_name = os.path.abspath(user_extra_path)
        selected_template_path = os.path.dirname(profile_tmp_name)
        selected_template_name = os.path.basename(profile_tmp_name)
        LOG.debug('Using user defined template path "%s"', profile_tmp_name)

    if os.path.isfile(profile_name):  # user direct path
        profile_tmp_name = os.path.abspath(profile_name)
        selected_template_path = os.path.dirname(profile_tmp_name)
        selected_template_name = os.path.basename(profile_tmp_name)
        LOG.debug('Using user defined template path "%s"', profile_tmp_name)

    complete_path = os.path.join(selected_template_path,
                                 selected_template_name)
    if not os.path.isfile(complete_path):
        raise ProfileError(
            'Unable to find a requested profile "%s"' % profile_name
        )
    return selected_template_name, selected_template_path


def select_template_dir(template_name):
    """Select template dir path (user defined or packaged)

    :param template_name: template name, from package, or dirname from user
    :type template_name: str

    :raises TemplateError: when requested template does not exists

    :return: selected path to template dir
    :rtype: str
    """
    templates_path = get_templates_path()
    selected_template_path = os.path.join(templates_path, template_name)
    user_extra_path = os.path.join(TEMPLATES, template_name)
    if os.path.isdir(user_extra_path):
        selected_template_path = user_extra_path
        LOG.debug('Using user defined template path "%s"', template_name)
    if os.path.isdir(template_name):
        selected_template_path = template_name
        LOG.debug('Using user defined template path "%s"', template_name)

    if not os.path.isdir(selected_template_path):
        raise TemplateError(
            'Unable to load requested template set "%s"' % template_name
        )

    if not os.path.isfile(os.path.join(selected_template_path, '_template')):
        raise TemplateError(
            'Selected template "%s" does not contain'
            ' "_template" file, so it is not considered a template'
        )
    LOG.debug('Selected template: %s', selected_template_path)
    return selected_template_path


def ensure_output_path(output_path):
    """Ensure that output path is actually existing directory

    :param output_path: selected output path to ensure
    :type output_path: str

    :raises OSError: if output_path exists but it is not a directory
    """
    if not os.path.isdir(output_path):
        if os.path.isfile(output_path):
            raise IOError(
                'Output path "%s" already exists and it is not a directory!'
                % output_path
            )
        os.makedirs(output_path, exist_ok=True)
        LOG.debug('Created directory "%s"', output_path)
    else:
        LOG.debug('Requested directory "%s" exists', output_path)


def get_output_filename(template_name):
    """Process a template filename and get output file name

    .. note: removing '.jinja2' suffix

    :param template_name: filename of a template
    :type template_name: str

    :return: output filename based on template name
    :rtype: str
    """
    match = REX_TEMPLATE_TO_OUTPUT.match(template_name)
    output_filename = template_name
    if match:
        output_filename = match.group(1)

    return output_filename



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import itertools
import logging
import os

import yaml
from jinja2 import Environment, FileSystemLoader

from .exceptions import ProfileError, TemplateError
from .files import select_profile_file, get_profiles_path

LOG = logging.getLogger(__name__)


def load_tuning_files(tuning_files=None):
    """Load tuning data from requested tuning files in order and
    provides list of tuning data for further processing.

    :param tuning_files: list of tuning files names
    :type tuning_files: list[str] | None

    :return: list of tuning data loaded from yaml tuning files
    :rtype: list[dict]
    """

    tuning_values_list = []

    if tuning_files:
        for tuning_file in tuning_files:
            try:
                tuning_values_list.append(yaml.load(stream=open(tuning_file, 'r'), Loader=yaml.SafeLoader))
            except IOError as exc:
                raise ProfileError(
                    'Unable to open tuning file "{}" {}'.format(
                        tuning_file, exc
                    )
                )
            except yaml.YAMLError as exc:
                raise ProfileError(
                    'Unable to parse YAML tuning file "{}" {}'.format(
                        tuning_files, exc
                    )
                )
            LOG.debug('Tuning file {} loaded', tuning_file)
    else:
        LOG.debug('No tuning files requested.')

    return tuning_values_list


def load_tuning(profile_defaults=None, tuning_files_list=None,
                tuning_data_list=None):
    """Load and apply all tuning, from profile defaults, from tuning
    files, and then directly provided tuning data. If provided.
    All data is applied in order.

    :param profile_defaults: profile defaults data to be tuned
    :type profile_defaults: dict | None
    :param tuning_files_list: list of tuning files names
    :type tuning_files_list: list[str] | None
    :param tuning_data_list: list of tuning data directly provided
    :type tuning_data_list: list[dict] | None

    :return: compound overlaid tuning data in order of appearance
    :rtype: dict
    """
    result = {}
    files_tuning_values = load_tuning_files(tuning_files_list)
    if profile_defaults:
        result.update(profile_defaults)
    if tuning_data_list is None:
        tuning_data_list = []

    for tuning_data in itertools.chain(files_tuning_values, tuning_data_list):
        result.update(tuning_data)

    return result


def get_tuned_profile(profile, tuning_files_list=None, tuning_data_list=None):
    """Get selected profile and use tuning data to fine tune
     it's variable values.

    :param profile: profile name (packaged) or path to profile
        (user specified)
    :type profile: str
    :param tuning_files_list: list of files with tuning data to be used
    :type tuning_files_list: list[str] | None
    :param tuning_data_list: data used to tune the variable values.
    :type tuning_data_list: list[dict] | None

    :raises ProfileError: when tuned profile is not valid.

    :return: compound tuned config data, and tuned profile yaml
    :rtype: dict, str
    """
    tuning_data = load_tuning(
        profile_defaults=load_profile_defaults(profile),
        tuning_files_list=tuning_files_list,
        tuning_data_list=tuning_data_list,
    )

    tuning_profile = get_profile_template(profile)
    tuning_data['profile_path'] = tuning_profile.name
    tuned_profile = tuning_profile.render(tuning_data)

    try:
        config_data = yaml.load(stream=tuned_profile, Loader=yaml.SafeLoader)
    except yaml.YAMLError as exc:
        raise ProfileError(
            'Unable to parse tuned profile "{}" {}'.format(
                profile, exc
            )
        )

    return config_data, tuned_profile


def load_profile_defaults(profile):
    """Load defaults variables from a profile if available

    .. note: profile will be rendered as scratch without any values
        to be able to be loaded as valid yaml.

    :param profile: profile name (from package or from user)
    :type profile: str

    :return: defaults values mapping, if not available then empty dict
    :rtype: dict
    """
    # scratch render of profile template for _defaults extraction
    scratch_profile = get_profile_template(profile)
    scratch_profile = scratch_profile.render()
    tmp_data = yaml.load(stream=scratch_profile, Loader=yaml.SafeLoader)
    tuning_data = tmp_data.get('_defaults', {})
    LOG.debug('Tuning data: %s', tuning_data)
    return tuning_data


def get_profile_template(profile_name):
    """Get a jinja2 template via env generated for selected profile
    (for fine-tuning of profile)

    :param profile_name: name of template set
        (alternatively path to user specified template set)
    :type profile_name: str

    :return: jinja2 profile template for fine tuning template
    :rtype: Template
    """

    selected_template_name, selected_template_path = \
        select_profile_file(profile_name)

    if not os.path.isdir(selected_template_path):
        raise TemplateError(
            'Unable to load requested profile location "%s"' % profile_name
        )

    env = Environment(
        loader=FileSystemLoader([
            selected_template_path,  # selected template
            get_profiles_path(),
        ]),
        trim_blocks=True,
        lstrip_blocks=True,
    )

    template = env.get_template(selected_template_name)

    return template



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import posixpath
import re

from .files import get_module_path
from .meta import TEMPLATES, PROFILES

LOG = logging.getLogger(__name__)


def filter_template_list(template_list, output_filter):
    """Based on list of output filters, select templates to be generated

    :param template_list: list of template file names
    :type template_list: list[str]
    :param output_filter: list of regular expressions to filter out templates
    :type output_filter: list[str]
    :return: list of selected template file names to process
    :rtype: list[str]
    """
    output_filter = [re.compile(flt) for flt in output_filter]
    template_list = [
        templ
        for templ in template_list
        for rex in output_filter if rex.match(templ)
    ]
    LOG.debug('Filtered template files list: %s', template_list)
    return template_list


def list_templates():
    """List all packaged templates with their package relative path

    Package is directory under 'templates' directory that contains
    file named '_template'. It would be good if template directory
    would contain some jinja2 templates.

    :return: list of templates paths from package
    :rtype: list[str]
    """
    module_path = get_module_path()

    templates_path = os.path.join(module_path, TEMPLATES)
    result = []

    for root, subdirs, files in os.walk(templates_path):
        for fn in files:
            if fn == '_template':
                prefix_path = os.path.relpath(root, templates_path)
                result.append(prefix_path)
                break

    result = [posixpath.join(*i.split(os.path.sep)) for i in result]

    return result


def list_profiles():
    """List all packaged complete profiles with their package relative path.

    Profile is any yaml file under 'profiles' directory in package, that is
    not placed under directory with leading underscore like _modules

    :return: list of profile from package
    :rtype: list[str]
    """
    module_path = get_module_path()
    profiles_path = os.path.join(module_path, PROFILES)

    result = []
    for root, _, files in os.walk(profiles_path):
        prefix_path = os.path.relpath(root, profiles_path)
        # skip over underscored paths
        path_levels = prefix_path.split(os.path.sep)
        if any([x.startswith('_') for x in path_levels]):
            continue
        # filter only yaml profiles
        tmp_files = [
            fn
            for fn in files if (
                fn.endswith('.yaml')
                or fn.endswith('.jinja2')
                or fn.endswith('.j2')
            )
        ]
        # add relative profile path if it is not profiles root,
        # it would add './' which is undesirable
        if prefix_path != '.':
            tmp_files = [
                os.path.join(prefix_path, fn)
                for fn in tmp_files
            ]
        result += tmp_files

    result = [posixpath.join(*i.split(os.path.sep)) for i in result]

    return result


def get_main_template_list(env):
    """Get list of main templates from selected template set

    .. note: main template -> template that resembles a output config file

    :param env: jinja2 environment
    :type env: Environment

    :return: list of main template names
    :rtype: list[str]
    """
    rex_main_template = re.compile(r'^[^/]+\.jinja2$')

    def main_template_filter(name):
        return rex_main_template.match(name)

    templ_list = env.list_templates(filter_func=main_template_filter)

    LOG.debug('Main template files list: %s', templ_list)

    return templ_list



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import ast
import copy
import json
import logging

import jinja2
from jinja2 import Environment, Template

from .config_data import add_template_metadata, add_render_config
from .exceptions import TemplateError, GenerationError
from .files import ensure_output_path, get_output_filename
from .output import write_output
from .profiles import get_tuned_profile
from .query import filter_template_list, get_main_template_list
from .templates import get_template_environment

# workaround for flake8: F401 'jinja2.Template' imported but unused
_t = Template
_e = Environment

LOG = logging.getLogger(__name__)


def generate_core(config_data, tuned_profile=None, template=None,
                  output_path=None, output_filter=None, render_options=None,
                  write_profile_data=False, extra_properties_data=None):
    """Core of the generator, gets complete dataset with selected
    template in config data or explicitly selected via template
    parameter at minimum, and generates outputs, if requested writes
    to file.

    :param config_data: complete and tuned config data
        or path to user provided profile
    :type config_data: dict
    :param tuned_profile: complete rendered yaml of tuned profile
        being used
    :type tuned_profile: str or None
    :param template: name of packaged template set,
        or path to user provided template set
    :type template: str or None
    :param output_path: proposed output path,
        if do not exists, it will be created
    :type output_path: str or None
    :param output_filter: list of regular expressions to filter out
        which output files should be generated, if None, then all will
        be generated, based on selected template set
    :type output_filter: list[str] or None
    :param render_options: extra render options tuning
    :type render_options: RenderOptions
    :param write_profile_data: enables writing profile data used for
        templating to file in a output path, output path must be
        specified
    :type write_profile_data: bool
    :param extra_properties_data: pass in any specific key/values
        and call filter, the filter will override the values
    :type extra_properties_data: dict[str, str]

    :return: mapping of filename to generated data for further use
    :rtype: dict[str, str] or dict[str, unicode]
    """
    add_template_metadata(config_data)
    if render_options:
        add_render_config(config_data, render_options)

    # template
    if template is None:
        template = config_data.get('render', {}).get('template')
        LOG.debug('Profile specified template: %s', template)
    if template is None:
        raise TemplateError(
            'Missing template. User nor profile specifies a template.'
        )

    env = get_template_environment(template)

    def override_value(value, value_key):
        """
        To work around some unexpected conversions by yaml loader
        for example empty string -> None and OFF -> False
        callers of yacfg need to pass in any specific key/values
        and call filter, the filter will override the values

        :param value:
        :type value: str
        :param value_key:
        :type value_key: str
        :return: str
        """
        if value_key in extra_properties_data:
            return extra_properties_data[value_key]
        return value

    def empty_filter(value, value_key):
        """
        Simply pass the value

        :param value:
        :type value: str
        :param value_key:
        :type value_key: str
        :return: str
        """
        return value

    def override_value_list_map_keys(value):
        """
        Replace keys with overrides if possible
        in list of maps
        :param value:
        :type value: list of dict
        :return: list of dict
        """
        for idx, item in enumerate(value):
            item = override_value_map_keys(item)
            value[idx] = item
        return value

    def override_value_map_keys(value):
        """
        Replace keys with overrides if possible
        :param value:
        :type value: dict
        :return: dict
        """
        new_map = dict()
        for key in value.keys():
            val = value[key]
            key = override_value(key, key)
            new_map[key] = val
        return new_map

    # Pass empty filter for performance if an extra_properties_data not defined (no more conditions)
    env.filters['overridevalue'] = override_value if extra_properties_data else empty_filter
    env.filters['overridevalue_listmapkeys'] = override_value_list_map_keys if extra_properties_data else empty_filter

    template_list = get_main_template_list(env)
    if output_filter:
        template_list = filter_template_list(template_list, output_filter)

    LOG.debug('Config data: %s', json.dumps(config_data))

    if output_path and tuned_profile:
        ensure_output_path(output_path)
        if write_profile_data:
            write_output('profile_data.yaml', output_path, tuned_profile)

    return generate_outputs(config_data, template_list, env, output_path)


def generate(profile, template=None, output_path=None,
             output_filter=None, render_options=None,
             tuning_files_list=None, tuning_data_list=None,
             write_profile_data=False, extra_properties_data=None):
    """Generate procedure using list of tuning data

    generate_via_tuning_files output files based on output_filter, from
    selected template set, with selected profile, and write output to
    a proposed output path.

    :param profile: name of packaged profile,
        or path to user provided profile
    :type profile: str
    :param template: name of packaged template set,
        or path to user provided template set
    :type template: str | None
    :param output_path: proposed output path,
        if do not exists, it will be created
    :type output_path: str | None
    :param output_filter: list of regular expressions to filter out
        which output files should be generated, if None, then all will
        be generated, based on selected template set
    :type output_filter: list[str] | None
    :param render_options: extra render options tuning
    :type render_options: RenderOptions
    :param tuning_files_list: Additional yaml tuning files with tuning
        values.
    :type tuning_files_list: list[str] | None
    :param tuning_data_list: Additional user values to fine-tune profile
        before applying it to template.
    :type tuning_data_list: list[dict] | None
    :param write_profile_data: enables writing profile data used for
        templating to file in a output path, output path must be
        specified
    :type write_profile_data: bool
    :param extra_properties_data: properties that can be used to help
        process templates with additional info
    :type extra_properties_data: dict[str,str]

    :return: mapping of filename to generated data for further use
    :rtype: dict[str, str] or dict[str, unicode]
    """

    config_data, tuned_profile = get_tuned_profile(
        profile=profile,
        tuning_files_list=tuning_files_list,
        tuning_data_list=tuning_data_list,
    )

    return generate_core(
        config_data=config_data,
        tuned_profile=tuned_profile,
        template=template,
        output_path=output_path,
        output_filter=output_filter,
        render_options=render_options,
        write_profile_data=write_profile_data,
        extra_properties_data=extra_properties_data
    )


# main alias
main = generate


def generate_outputs(config_data, template_list, env, output_path=None):
    """Generate output files based on config_data, (filtered) template list,
    within provided jinja environment, and if output_path is specified, then
    write results to that directory.

    .. note: output_path directory has to be created before.

    :param config_data: configuration data mapping for templating
    :type config_data: dict
    :param template_list: list of main template file names
    :type template_list: list[str]
    :param env: jinja2 template environment
    :type env: Environment
    :param output_path: path where to generate_via_tuning_files output files,
        or None to do a dry run
    :type output_path: str

    :raises GenerationError: when there was a problem with generating one of
        config files

    :return: mapping of filename to generated data for further use
    :rtype: dict[str, str] or dict[str, unicode]
    """
    generate_exception = None
    result_data = {}

    # only additions of runtime data happens on base level,
    # so deeper structures are unaffected
    config_data = copy.copy(config_data)

    # metadata structure initialization,
    # if called without add_template_metadata()
    if 'metadata' not in config_data:
        config_data['metadata'] = {}

    for template_name in template_list:
        out_filename = get_output_filename(template_name)
        config_data['metadata']['out_filename'] = template_name

        try:
            template = env.get_template(template_name)
            output_data = template.render(config_data)
        except jinja2.TemplateError as exc:
            LOG.error('Config file "%s" generation FAILED', out_filename)
            LOG.exception('Original error')
            if not generate_exception:
                generate_exception = GenerationError(
                    'There was a problem generating file "%s" with "%s" '
                    'template: %s'
                    % (out_filename, template_name, exc)
                )
        else:
            LOG.debug('BEGIN %s\n%s', out_filename, output_data)
            LOG.debug('END %s' % out_filename)
            LOG.info('Config file "%s" generation PASSED', out_filename)

            result_data[out_filename] = output_data

            if output_path:
                write_output(out_filename, output_path, output_data)

    if generate_exception:
        raise generate_exception

    return result_data



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import logging
import os
import sys

from .config_data import RenderOptions
from .yacfg import generate
from .cli_arguments import parser, boolize, parse_key_value_list
from .exceptions import TemplateError, ProfileError, GenerationError
from .meta import NAME
from .output import (
    new_profile, new_profile_rendered,
    new_template, export_tuning_variables
)
from .query import list_templates, list_profiles

from . import logger_settings
from . import __version__

logger_settings.config_console_logger()

LOG = logging.getLogger(NAME)


def error(msg, ecode=2):
    LOG.error(msg)
    if ecode != 0:
        sys.exit(ecode)


def main():
    options = parser.parse_args()

    # logging adjustments
    root_logger = logging.getLogger()
    if options.verbose:
        root_logger.setLevel(logging.INFO)
    if options.quiet:
        root_logger.setLevel(logging.ERROR)
    if options.debug:
        root_logger.setLevel(logging.DEBUG)

    # post-process direct options
    if options.opt:
        # list is because generate api expects list of tuning dicts
        try:
            # false negative, there is not type mismatch, ArgParse
            # does not parse KEY=VALUE pairs automatically
            # noinspection PyTypeChecker
            options.opt = [parse_key_value_list(options.opt)]
        except ValueError as exc:
            error(exc, 2)

    LOG.debug('Direct Tuning options %s', options.opt)

    do_not_generate = False

    if options.version:
        print(__version__)
        return

    if options.list_templates:
        LOG.info('Available Templates:')
        print(os.linesep.join(list_templates()))
        return

    if options.list_profiles:
        LOG.info('Available Profiles:')
        print(os.linesep.join(list_profiles()))
        return

    error_do_not_generate = False

    if options.new_profile or options.new_profile_static:
        do_not_generate = True
        if not options.profile:
            error('Missing parameters profile', 0)
            error_do_not_generate = True
        else:
            try:
                if options.new_profile:
                    new_profile(options.profile, options.new_profile)
                if options.new_profile_static:
                    new_profile_rendered(
                        profile=options.profile,
                        dest_profile=options.new_profile_static,
                        tuning_files=options.tune,
                        tuning_data_list=options.opt,
                    )
            except (ProfileError, IOError, OSError) as exc:
                error(exc, 0)
                error_do_not_generate = True

    if options.export_tuning:
        do_not_generate = True
        if not options.profile:
            error('Missing parameters profile', 0)
            error_do_not_generate = True
        else:
            try:
                export_tuning_variables(options.profile, options.export_tuning)
            except (ProfileError, IOError, OSError) as exc:
                error(exc, 0)
                error_do_not_generate = True

    if options.new_template:
        do_not_generate = True
        if not options.template:
            error('Missing parameter template, cannot export', 0)
            error_do_not_generate = True
        else:
            try:
                new_template(options.template, options.new_template)
            except (TemplateError, IOError, OSError) as exc:
                error(exc, 0)
                error_do_not_generate = True

    if error_do_not_generate:
        error('Cannot continue due to existing above problems', 2)

    # Generator barrier for actions that prevents generation
    if do_not_generate:
        sys.exit(0)

    if not options.profile:
        error('Missing parameters profile', 0)
        error_do_not_generate = True

    if error_do_not_generate:
        # parser.print_help()
        error('Cannot continue due to existing above problems', 2)

    # Tune up render options
    render_options = RenderOptions(
        boolize(options.render_generator_notice),
        boolize(options.render_licenses),
    )

    try:
        generate(
            profile=options.profile,
            template=options.template,
            output_path=options.output,
            output_filter=options.filter,
            render_options=render_options,
            tuning_files_list=options.tune,
            tuning_data_list=options.opt,
            write_profile_data=options.save_effective_profile,
            extra_properties_data=options.extra_properties
        )
    except TemplateError as exc:
        error(exc)
    except ProfileError as exc:
        error(exc)
    except GenerationError as exc:
        error(exc, 1)


if __name__ == '__main__':
    main()



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import sys

FORMAT_CONSOLE = '[%(asctime)s] [%(levelname)s] %(name)s :: %(message)s'
FORMAT_CONSOLE_COLOR = '[%(asctime)s] [%(log_color)s%(levelname)s%(reset)s]' \
                       ' %(name)s :: %(message)s'

DATE_FORMAT_ISO = '%Y-%m-%d %H:%M:%S,'
DATE_FORMAT_TIME = '%H:%M:%S'


def config_console_logger(filename=None, level=None, fmt=None, fmt_color=None,
                          datefmt=None):
    """ configure console logger, if colorlog is installed it will use colorlog
    :param filename: stream to log into, default sys.stdout
    :param level: console log level
    :param fmt: console log format
    :param fmt_color: console color log format (only with colorlog)
    :param datefmt: formatter date format, default like ISO8601
    """
    import_error_msg = []
    # Windows compatibility section for colorama
    try:
        # noinspection PyUnresolvedReferences
        import colorama
        colorama.init()
    except ImportError:
        try:
            # noinspection PyUnresolvedReferences
            import colorlog
        except ImportError:
            import platform
            if platform.system() == 'Windows':
                import_error_msg.append(
                    'Colorlog without colorama used, expect escape codes,'
                    ' please install colorama'
                )

    filename = filename or sys.stdout
    level = level or logging.DEBUG
    fmt = fmt or FORMAT_CONSOLE
    fmt_color = fmt_color or FORMAT_CONSOLE_COLOR
    datefmt = datefmt or DATE_FORMAT_TIME

    # console handler
    # console_handler = logging.StreamHandler(stream=filename)  # Python 2.7
    console_handler = logging.StreamHandler()  # Python 2.6
    console_handler.setLevel(level)
    console_formatter = logging.Formatter(fmt, datefmt=datefmt)

    try:
        # noinspection PyUnresolvedReferences
        import colorlog  # noqa: F811
        console_formatter = colorlog.ColoredFormatter(
            fmt_color,
            log_colors={
                'CRITICAL': 'black,bg_red',
                'ERROR': 'red',
                'WARNING': 'yellow',
                'SKIP': 'blue',
                'FAIL': 'bold_red',
                'PASS': 'bold_green',
                'INFO': 'reset',
                'DOC': 'purple',
                'DEBUG': 'cyan',
            },
            datefmt=datefmt,
        )
    except ImportError:
        # import_error_msg.append(
        #     "cannot initialize colorlog, using boring streamhandler"
        # )
        pass
    console_handler.setFormatter(console_formatter)
    logging.getLogger().addHandler(console_handler)
    # log all above error messages
    if import_error_msg:
        for log_err in import_error_msg:
            logging.info(log_err)


# default logger level INFO
logging.getLogger().setLevel(logging.INFO)



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import logging
import sys

from yacfg import logger_settings
from yacfg.meta import NAME
from .yacfg_batch import generate
from .cli_arguments import parser
from . import __version__

logger_settings.config_console_logger()

LOG = logging.getLogger(NAME)


def error(msg, ecode=2):
    LOG.error(msg)
    if ecode != 0:
        sys.exit(ecode)


def main():
    options = parser.parse_args()

    # logging adjustments
    root_logger = logging.getLogger()
    if options.verbose:
        root_logger.setLevel(logging.INFO)
    if options.quiet:
        root_logger.setLevel(logging.ERROR)
    if options.debug:
        root_logger.setLevel(logging.DEBUG)

    if options.version:
        print(__version__)
        return

    if not options.input:
        error(
            'Missing parameter input, cannot work without input.',
            2
        )

    if options.input:
        generate(options.input, options.output)

    print('have a nice day.')


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['yacfg', 'yacfg_batch']

package_data = \
{'': ['*'],
 'yacfg': ['profiles/_libs/*',
           'profiles/amq_broker/7.2.0/*',
           'profiles/amq_broker/7.2.0/_modules/bootstrap_xml/*',
           'profiles/amq_broker/7.2.0/cluster/*',
           'profiles/amq_broker/7.2.0/ha/*',
           'profiles/amq_broker/7.2.0/performance/*',
           'profiles/amq_broker/7.2.0/security/gssapi/*',
           'profiles/amq_broker/7.2.0/security/gssapi_ldap/*',
           'profiles/amq_broker/7.2.0/security/ssl/*',
           'profiles/amq_broker/7.2.0/store/jdbc/*',
           'profiles/amq_broker/7.4.0/*',
           'profiles/amq_broker/7.4.0/_modules/bootstrap_xml/*',
           'profiles/amq_broker/7.4.0/_modules/broker_xml/store/*',
           'profiles/amq_broker/7.4.0/cluster/*',
           'profiles/amq_broker/7.4.0/ha/*',
           'profiles/amq_broker/7.4.0/performance/*',
           'profiles/amq_broker/7.4.0/security/gssapi/*',
           'profiles/amq_broker/7.4.0/security/gssapi_ldap/*',
           'profiles/amq_broker/7.4.0/store/jdbc/*',
           'profiles/amq_broker/7.6.0/*',
           'profiles/amq_broker/7.6.0/_modules/bootstrap_xml/*',
           'profiles/amq_broker/7.6.0/_modules/broker_xml/store/*',
           'profiles/amq_broker/7.6.0/cluster/*',
           'profiles/amq_broker/7.6.0/ha/*',
           'profiles/amq_broker/7.6.0/performance/*',
           'profiles/amq_broker/7.6.0/security/gssapi/*',
           'profiles/amq_broker/7.6.0/security/gssapi_ldap/*',
           'profiles/amq_broker/7.6.0/store/jdbc/*',
           'profiles/amq_broker/7.7.0/*',
           'profiles/amq_broker/7.7.0/_modules/bootstrap_xml/*',
           'profiles/amq_broker/7.7.0/_modules/broker_xml/store/*',
           'profiles/amq_broker/7.7.0/cluster/*',
           'profiles/amq_broker/7.7.0/ha/*',
           'profiles/amq_broker/7.7.0/performance/*',
           'profiles/amq_broker/7.7.0/security/gssapi/*',
           'profiles/amq_broker/7.7.0/security/gssapi_ldap/*',
           'profiles/amq_broker/7.7.0/store/jdbc/*',
           'profiles/amq_broker/7.8.0/*',
           'profiles/amq_broker/7.8.0/_modules/bootstrap_xml/*',
           'profiles/amq_broker/7.8.0/_modules/broker_xml/store/*',
           'profiles/amq_broker/7.8.0/cluster/*',
           'profiles/amq_broker/7.8.0/ha/*',
           'profiles/amq_broker/7.8.0/performance/*',
           'profiles/amq_broker/7.8.0/security/gssapi/*',
           'profiles/amq_broker/7.8.0/security/gssapi_ldap/*',
           'profiles/amq_broker/7.8.0/store/jdbc/*',
           'profiles/amq_broker/7.8.1/*',
           'profiles/amq_broker/7.8.1/_modules/bootstrap_xml/*',
           'profiles/amq_broker/7.8.1/_modules/broker_xml/store/*',
           'profiles/amq_broker/7.8.1/cluster/*',
           'profiles/amq_broker/7.8.1/ha/*',
           'profiles/amq_broker/7.8.1/performance/*',
           'profiles/amq_broker/7.8.1/security/gssapi/*',
           'profiles/amq_broker/7.8.1/security/gssapi_ldap/*',
           'profiles/amq_broker/7.8.1/store/jdbc/*',
           'profiles/amq_broker/7.8.2/*',
           'profiles/amq_broker/7.8.2/_modules/bootstrap_xml/*',
           'profiles/amq_broker/7.8.2/_modules/broker_xml/store/*',
           'profiles/amq_broker/7.8.2/cluster/*',
           'profiles/amq_broker/7.8.2/ha/*',
           'profiles/amq_broker/7.8.2/performance/*',
           'profiles/amq_broker/7.8.2/security/gssapi/*',
           'profiles/amq_broker/7.8.2/security/gssapi_ldap/*',
           'profiles/amq_broker/7.8.2/store/jdbc/*',
           'profiles/artemis/2.13.0/*',
           'profiles/artemis/2.13.0/_modules/artemis_profile/*',
           'profiles/artemis/2.13.0/_modules/artemis_profile/performance/*',
           'profiles/artemis/2.13.0/_modules/bootstrap_xml/*',
           'profiles/artemis/2.13.0/_modules/broker_xml/*',
           'profiles/artemis/2.13.0/_modules/broker_xml/acceptors/*',
           'profiles/artemis/2.13.0/_modules/broker_xml/address_settings/*',
           'profiles/artemis/2.13.0/_modules/broker_xml/addresses/*',
           'profiles/artemis/2.13.0/_modules/broker_xml/cluster/*',
           'profiles/artemis/2.13.0/_modules/broker_xml/ha_policy/replication/*',
           'profiles/artemis/2.13.0/_modules/broker_xml/ha_policy/shared_store/*',
           'profiles/artemis/2.13.0/_modules/broker_xml/journal/*',
           'profiles/artemis/2.13.0/_modules/broker_xml/store/*',
           'profiles/artemis/2.13.0/_modules/jolokia_access/*',
           'profiles/artemis/2.13.0/_modules/logging_properties/*',
           'profiles/artemis/2.13.0/_modules/login_config/*',
           'profiles/artemis/2.13.0/_modules/management_xml/*',
           'profiles/artemis/2.13.0/_modules/security/jdk_gssapi/*',
           'profiles/artemis/2.13.0/_modules/users_roles_security/*',
           'profiles/artemis/2.13.0/cluster/*',
           'profiles/artemis/2.13.0/ha/*',
           'profiles/artemis/2.13.0/performance/*',
           'profiles/artemis/2.13.0/security/gssapi/*',
           'profiles/artemis/2.13.0/security/gssapi/ssl_tls/*',
           'profiles/artemis/2.13.0/security/gssapi_ldap/*',
           'profiles/artemis/2.13.0/store/jdbc/*',
           'profiles/artemis/2.14.0/*',
           'profiles/artemis/2.14.0/_modules/artemis_profile/*',
           'profiles/artemis/2.14.0/_modules/artemis_profile/performance/*',
           'profiles/artemis/2.14.0/_modules/bootstrap_xml/*',
           'profiles/artemis/2.14.0/_modules/broker_xml/*',
           'profiles/artemis/2.14.0/_modules/broker_xml/acceptors/*',
           'profiles/artemis/2.14.0/_modules/broker_xml/address_settings/*',
           'profiles/artemis/2.14.0/_modules/broker_xml/addresses/*',
           'profiles/artemis/2.14.0/_modules/broker_xml/cluster/*',
           'profiles/artemis/2.14.0/_modules/broker_xml/ha_policy/replication/*',
           'profiles/artemis/2.14.0/_modules/broker_xml/ha_policy/shared_store/*',
           'profiles/artemis/2.14.0/_modules/broker_xml/journal/*',
           'profiles/artemis/2.14.0/_modules/broker_xml/store/*',
           'profiles/artemis/2.14.0/_modules/jolokia_access/*',
           'profiles/artemis/2.14.0/_modules/logging_properties/*',
           'profiles/artemis/2.14.0/_modules/login_config/*',
           'profiles/artemis/2.14.0/_modules/management_xml/*',
           'profiles/artemis/2.14.0/_modules/security/jdk_gssapi/*',
           'profiles/artemis/2.14.0/_modules/users_roles_security/*',
           'profiles/artemis/2.14.0/cluster/*',
           'profiles/artemis/2.14.0/ha/*',
           'profiles/artemis/2.14.0/performance/*',
           'profiles/artemis/2.14.0/security/gssapi/*',
           'profiles/artemis/2.14.0/security/gssapi/ssl_tls/*',
           'profiles/artemis/2.14.0/security/gssapi_ldap/*',
           'profiles/artemis/2.14.0/store/jdbc/*',
           'profiles/artemis/2.15.0/*',
           'profiles/artemis/2.15.0/_modules/artemis_profile/*',
           'profiles/artemis/2.15.0/_modules/artemis_profile/performance/*',
           'profiles/artemis/2.15.0/_modules/bootstrap_xml/*',
           'profiles/artemis/2.15.0/_modules/broker_xml/*',
           'profiles/artemis/2.15.0/_modules/broker_xml/acceptors/*',
           'profiles/artemis/2.15.0/_modules/broker_xml/address_settings/*',
           'profiles/artemis/2.15.0/_modules/broker_xml/addresses/*',
           'profiles/artemis/2.15.0/_modules/broker_xml/cluster/*',
           'profiles/artemis/2.15.0/_modules/broker_xml/ha_policy/replication/*',
           'profiles/artemis/2.15.0/_modules/broker_xml/ha_policy/shared_store/*',
           'profiles/artemis/2.15.0/_modules/broker_xml/journal/*',
           'profiles/artemis/2.15.0/_modules/broker_xml/store/*',
           'profiles/artemis/2.15.0/_modules/jolokia_access/*',
           'profiles/artemis/2.15.0/_modules/logging_properties/*',
           'profiles/artemis/2.15.0/_modules/login_config/*',
           'profiles/artemis/2.15.0/_modules/management_xml/*',
           'profiles/artemis/2.15.0/_modules/security/jdk_gssapi/*',
           'profiles/artemis/2.15.0/_modules/users_roles_security/*',
           'profiles/artemis/2.15.0/cluster/*',
           'profiles/artemis/2.15.0/ha/*',
           'profiles/artemis/2.15.0/performance/*',
           'profiles/artemis/2.15.0/security/gssapi/*',
           'profiles/artemis/2.15.0/security/gssapi/ssl_tls/*',
           'profiles/artemis/2.15.0/security/gssapi_ldap/*',
           'profiles/artemis/2.15.0/store/jdbc/*',
           'profiles/artemis/2.16.0/*',
           'profiles/artemis/2.16.0/_modules/artemis_profile/*',
           'profiles/artemis/2.16.0/_modules/artemis_profile/performance/*',
           'profiles/artemis/2.16.0/_modules/bootstrap_xml/*',
           'profiles/artemis/2.16.0/_modules/broker_xml/*',
           'profiles/artemis/2.16.0/_modules/broker_xml/acceptors/*',
           'profiles/artemis/2.16.0/_modules/broker_xml/address_settings/*',
           'profiles/artemis/2.16.0/_modules/broker_xml/addresses/*',
           'profiles/artemis/2.16.0/_modules/broker_xml/cluster/*',
           'profiles/artemis/2.16.0/_modules/broker_xml/ha_policy/replication/*',
           'profiles/artemis/2.16.0/_modules/broker_xml/ha_policy/shared_store/*',
           'profiles/artemis/2.16.0/_modules/broker_xml/journal/*',
           'profiles/artemis/2.16.0/_modules/broker_xml/store/*',
           'profiles/artemis/2.16.0/_modules/jolokia_access/*',
           'profiles/artemis/2.16.0/_modules/logging_properties/*',
           'profiles/artemis/2.16.0/_modules/login_config/*',
           'profiles/artemis/2.16.0/_modules/management_xml/*',
           'profiles/artemis/2.16.0/_modules/security/jdk_gssapi/*',
           'profiles/artemis/2.16.0/_modules/users_roles_security/*',
           'profiles/artemis/2.16.0/cluster/*',
           'profiles/artemis/2.16.0/ha/*',
           'profiles/artemis/2.16.0/performance/*',
           'profiles/artemis/2.16.0/security/gssapi/*',
           'profiles/artemis/2.16.0/security/gssapi/ssl_tls/*',
           'profiles/artemis/2.16.0/security/gssapi_ldap/*',
           'profiles/artemis/2.16.0/store/jdbc/*',
           'profiles/artemis/2.17.0/*',
           'profiles/artemis/2.17.0/_modules/artemis_profile/*',
           'profiles/artemis/2.17.0/_modules/artemis_profile/performance/*',
           'profiles/artemis/2.17.0/_modules/bootstrap_xml/*',
           'profiles/artemis/2.17.0/_modules/broker_xml/*',
           'profiles/artemis/2.17.0/_modules/broker_xml/acceptors/*',
           'profiles/artemis/2.17.0/_modules/broker_xml/address_settings/*',
           'profiles/artemis/2.17.0/_modules/broker_xml/addresses/*',
           'profiles/artemis/2.17.0/_modules/broker_xml/cluster/*',
           'profiles/artemis/2.17.0/_modules/broker_xml/ha_policy/replication/*',
           'profiles/artemis/2.17.0/_modules/broker_xml/ha_policy/shared_store/*',
           'profiles/artemis/2.17.0/_modules/broker_xml/journal/*',
           'profiles/artemis/2.17.0/_modules/broker_xml/store/*',
           'profiles/artemis/2.17.0/_modules/jolokia_access/*',
           'profiles/artemis/2.17.0/_modules/logging_properties/*',
           'profiles/artemis/2.17.0/_modules/login_config/*',
           'profiles/artemis/2.17.0/_modules/management_xml/*',
           'profiles/artemis/2.17.0/_modules/security/jdk_gssapi/*',
           'profiles/artemis/2.17.0/_modules/users_roles_security/*',
           'profiles/artemis/2.17.0/cluster/*',
           'profiles/artemis/2.17.0/ha/*',
           'profiles/artemis/2.17.0/performance/*',
           'profiles/artemis/2.17.0/security/gssapi/*',
           'profiles/artemis/2.17.0/security/gssapi/ssl_tls/*',
           'profiles/artemis/2.17.0/security/gssapi_ldap/*',
           'profiles/artemis/2.17.0/store/jdbc/*',
           'profiles/artemis/2.5.0/*',
           'profiles/artemis/2.5.0/_modules/artemis_profile/*',
           'profiles/artemis/2.5.0/_modules/artemis_profile/performance/*',
           'profiles/artemis/2.5.0/_modules/bootstrap_xml/*',
           'profiles/artemis/2.5.0/_modules/broker_xml/*',
           'profiles/artemis/2.5.0/_modules/broker_xml/acceptors/*',
           'profiles/artemis/2.5.0/_modules/broker_xml/address_settings/*',
           'profiles/artemis/2.5.0/_modules/broker_xml/addresses/*',
           'profiles/artemis/2.5.0/_modules/broker_xml/cluster/*',
           'profiles/artemis/2.5.0/_modules/broker_xml/ha_policy/replication/*',
           'profiles/artemis/2.5.0/_modules/broker_xml/ha_policy/shared_store/*',
           'profiles/artemis/2.5.0/_modules/broker_xml/journal/*',
           'profiles/artemis/2.5.0/_modules/broker_xml/store/*',
           'profiles/artemis/2.5.0/_modules/jolokia_access/*',
           'profiles/artemis/2.5.0/_modules/logging_properties/*',
           'profiles/artemis/2.5.0/_modules/login_config/*',
           'profiles/artemis/2.5.0/_modules/management_xml/*',
           'profiles/artemis/2.5.0/_modules/security/jdk_gssapi/*',
           'profiles/artemis/2.5.0/_modules/users_roles_security/*',
           'profiles/artemis/2.5.0/cluster/*',
           'profiles/artemis/2.5.0/ha/*',
           'profiles/artemis/2.5.0/performance/*',
           'profiles/artemis/2.5.0/security/gssapi/*',
           'profiles/artemis/2.5.0/store/jdbc/*',
           'profiles/artemis/2.6.0/*',
           'profiles/artemis/2.6.0/_modules/artemis_profile/*',
           'profiles/artemis/2.6.0/_modules/artemis_profile/performance/*',
           'profiles/artemis/2.6.0/_modules/bootstrap_xml/*',
           'profiles/artemis/2.6.0/_modules/broker_xml/*',
           'profiles/artemis/2.6.0/_modules/broker_xml/acceptors/*',
           'profiles/artemis/2.6.0/_modules/broker_xml/address_settings/*',
           'profiles/artemis/2.6.0/_modules/broker_xml/addresses/*',
           'profiles/artemis/2.6.0/_modules/broker_xml/cluster/*',
           'profiles/artemis/2.6.0/_modules/broker_xml/ha_policy/replication/*',
           'profiles/artemis/2.6.0/_modules/broker_xml/ha_policy/shared_store/*',
           'profiles/artemis/2.6.0/_modules/broker_xml/journal/*',
           'profiles/artemis/2.6.0/_modules/broker_xml/store/*',
           'profiles/artemis/2.6.0/_modules/jolokia_access/*',
           'profiles/artemis/2.6.0/_modules/logging_properties/*',
           'profiles/artemis/2.6.0/_modules/login_config/*',
           'profiles/artemis/2.6.0/_modules/management_xml/*',
           'profiles/artemis/2.6.0/_modules/security/jdk_gssapi/*',
           'profiles/artemis/2.6.0/_modules/users_roles_security/*',
           'profiles/artemis/2.6.0/cluster/*',
           'profiles/artemis/2.6.0/ha/*',
           'profiles/artemis/2.6.0/performance/*',
           'profiles/artemis/2.6.0/security/gssapi/*',
           'profiles/artemis/2.6.0/security/gssapi/ssl_tls/*',
           'profiles/artemis/2.6.0/security/gssapi_ldap/*',
           'profiles/artemis/2.6.0/store/jdbc/*',
           'profiles/artemis/2.9.0/*',
           'profiles/artemis/2.9.0/_modules/artemis_profile/*',
           'profiles/artemis/2.9.0/_modules/artemis_profile/performance/*',
           'profiles/artemis/2.9.0/_modules/bootstrap_xml/*',
           'profiles/artemis/2.9.0/_modules/broker_xml/*',
           'profiles/artemis/2.9.0/_modules/broker_xml/acceptors/*',
           'profiles/artemis/2.9.0/_modules/broker_xml/address_settings/*',
           'profiles/artemis/2.9.0/_modules/broker_xml/addresses/*',
           'profiles/artemis/2.9.0/_modules/broker_xml/cluster/*',
           'profiles/artemis/2.9.0/_modules/broker_xml/ha_policy/replication/*',
           'profiles/artemis/2.9.0/_modules/broker_xml/ha_policy/shared_store/*',
           'profiles/artemis/2.9.0/_modules/broker_xml/journal/*',
           'profiles/artemis/2.9.0/_modules/broker_xml/store/*',
           'profiles/artemis/2.9.0/_modules/jolokia_access/*',
           'profiles/artemis/2.9.0/_modules/logging_properties/*',
           'profiles/artemis/2.9.0/_modules/login_config/*',
           'profiles/artemis/2.9.0/_modules/management_xml/*',
           'profiles/artemis/2.9.0/_modules/security/jdk_gssapi/*',
           'profiles/artemis/2.9.0/_modules/users_roles_security/*',
           'profiles/artemis/2.9.0/cluster/*',
           'profiles/artemis/2.9.0/ha/*',
           'profiles/artemis/2.9.0/performance/*',
           'profiles/artemis/2.9.0/security/gssapi/*',
           'profiles/artemis/2.9.0/security/gssapi/ssl_tls/*',
           'profiles/artemis/2.9.0/security/gssapi_ldap/*',
           'profiles/artemis/2.9.0/store/jdbc/*',
           'templates/artemis/2.13.0/*',
           'templates/artemis/2.13.0/modules/broker_xml/*',
           'templates/artemis/2.13.0/modules/broker_xml/ha_policy/*',
           'templates/artemis/2.13.0/modules/broker_xml/security_settings/*',
           'templates/artemis/2.13.0/modules/broker_xml/store/*',
           'templates/artemis/2.14.0/*',
           'templates/artemis/2.14.0/modules/broker_xml/*',
           'templates/artemis/2.14.0/modules/broker_xml/ha_policy/*',
           'templates/artemis/2.14.0/modules/broker_xml/security_settings/*',
           'templates/artemis/2.14.0/modules/broker_xml/store/*',
           'templates/artemis/2.15.0/*',
           'templates/artemis/2.15.0/modules/broker_xml/*',
           'templates/artemis/2.15.0/modules/broker_xml/ha_policy/*',
           'templates/artemis/2.15.0/modules/broker_xml/security_settings/*',
           'templates/artemis/2.15.0/modules/broker_xml/store/*',
           'templates/artemis/2.16.0/*',
           'templates/artemis/2.16.0/modules/broker_xml/*',
           'templates/artemis/2.16.0/modules/broker_xml/ha_policy/*',
           'templates/artemis/2.16.0/modules/broker_xml/security_settings/*',
           'templates/artemis/2.16.0/modules/broker_xml/store/*',
           'templates/artemis/2.17.0/*',
           'templates/artemis/2.17.0/modules/broker_xml/*',
           'templates/artemis/2.17.0/modules/broker_xml/ha_policy/*',
           'templates/artemis/2.17.0/modules/broker_xml/security_settings/*',
           'templates/artemis/2.17.0/modules/broker_xml/store/*',
           'templates/artemis/2.5.0/*',
           'templates/artemis/2.5.0/modules/broker_xml/*',
           'templates/artemis/2.5.0/modules/broker_xml/ha_policy/*',
           'templates/artemis/2.5.0/modules/broker_xml/security_settings/*',
           'templates/artemis/2.5.0/modules/broker_xml/store/*',
           'templates/artemis/2.6.0/*',
           'templates/artemis/2.6.0/modules/broker_xml/*',
           'templates/artemis/2.6.0/modules/broker_xml/ha_policy/*',
           'templates/artemis/2.6.0/modules/broker_xml/security_settings/*',
           'templates/artemis/2.6.0/modules/broker_xml/store/*',
           'templates/artemis/2.9.0/*',
           'templates/artemis/2.9.0/modules/broker_xml/*',
           'templates/artemis/2.9.0/modules/broker_xml/ha_policy/*',
           'templates/artemis/2.9.0/modules/broker_xml/security_settings/*',
           'templates/artemis/2.9.0/modules/broker_xml/store/*',
           'templates/libs/*',
           'templates/libs/headers/*',
           'templates/libs/licenses/apache-2.0/*']}

install_requires = \
['jinja2==2.11.3', 'pyaml==20.4.0']

extras_require = \
{'color_log': ['colorlog>=5.0.1,<6.0.0', 'colorama>=0.4.4,<0.5.0']}

entry_points = \
{'console_scripts': ['yacfg = yacfg.yacfg_cli:main',
                     'yacfg-batch = yacfg_batch.yacfg_batch_cli:main']}

setup_kwargs = {
    'name': 'yacfg',
    'version': '0.9.0',
    'description': 'Template based configuration files generator based on jinja2 and yaml.',
    'long_description': '# yacfg - YAML Configurator\n\nThis tool can generate a set of configuration files mainly needed for\nAMQ Broker, but it is not limited to only generating files for one product.\n\nIt has a user facing Command Line Tool for quick and easy command line usage.\nFurthermore, it is possible to use its API in your python code.\n\n## Getting started\n\n* Python 3.5+\n* Python Poetry\n\n### From git\n\n```bash\ngit clone git@github.com:rh-messaging-qe/yacfg.git\npoetry install\nyacfg --help\n```\n\n### From PyPI\n\n```bash\npip install yacfg\nyacfg --help\n```\n\n## User (CLI) guide\n\n```bash\nyacfg --help\n\nyacfg --list-profiles\nyacfg --list-templates\n\n# perform a generation of a default profile\nyacfg --profile artemis/2.5.0/default.yaml.jinja2\n# also save result to [OUTDIR] directory\nyacfg --profile [PROFILE] --output [OUTDIR]\n```\n\n## Customization\n\nQuickest way to customize data is to use hot-variables, basically variables\nthat the profile itself provides for tuning. Next step is to write (modify) custom\nprofile with completely custom values.\nIf that does not satisfy your needs, then a custom template might be required.\n\n### Profile tuning\n\nSimply export tuning values from profile you want to tune and change those you\nneed to change. Then supply the custom tuning file(s) when generating the profile.\n\n```bash\nyacfg --profile [PROFILE] --export-tuning my_values.yaml\nvim my_values.yaml\nyacfg --profile [PROFILE] --tune my_values.yaml\n\n# multiple tuning files can be overlaid\n# they are updated in sequence, only values present are overwritten\nyacfg --profile [PROFILE] --tune my_values.yaml --tune machine_specific.yaml \\\n       --tune logging_debug.yaml --output [OUTDIR]\n```\n\n## Custom profiles\n\nWrite your own, or simply export an existing profile and modify that.\n\nYou can export dynamic version with includes of some modules, that would still\n work. Either you can use imports from package, or your own local files.\n\nOr you can export completely rendered profile file without any includes or\nvariables and modify that as you like.\n\n\n```bash\n# export profile with dynamic includes still active jinja2 files\nyacfg --profile [PROFILE] --new-profile my_new_profile.yaml.jinja2\n# export completely generated profile without any jinja2 fields, plain yaml\nyacfg --profile [PROFILE] --new-profile-static my_new_profile.yaml\nvim my_new_profile.yaml\nyacfg --profile my_new_profile.yaml\n```\n\nProfile is just another jinja2 file that enables customization of profile data\n -- that is tuning. Becuase of that we recommend to keep the extension `.yaml.jinja2`\n unless it is static profile withou any jinja2 capabilities, in that case it could\n be named `.yaml`. That way we can run yaml lint against static profiles and verify\n that they are correct.\n\n All profiles have to be used to generate files without any tuning. That means,\n if they are tune-able, they have to contain all default values in `_defaults` section.\n That section is also used for tuning, so any variable in there will be exported as tuning.\n\n## Custom templates\n\nThe last resort is to export a template and modify that. But remember a template,\nor more correctly a template set is a directory containing a set of main\ntemplates that subsequently generate_via_tuning_files a new file.\n\nOf course feel free to write your own templates. Especially when you need to\ngenerate_via_tuning_files files for something that is not packaged.\n\nJust remember for a template set to be identified the directory must contain\na file named \'_template\' and then main templates ending with \'.jinja2\'.\n\n```bash\nyacfg --template [TEMPLATE] --new-template my_new_template\nvim my_new_template/[MAIN_TEMPLATES].jinja2\nyacfg --template my_new_template --profile [PROFILE]\n\n```\n\n## API guide\n\nDirect use of API is to use `generate()` nearly the same as the CLI.\nWith option to use tuning values directly.\n\nTuning data will be overlaid in order of appearance, using python\ndict.update(), so values that will appear later will overwrite previous\nvalues. We recommend that tuning values are always flat, because update\nis not recursive. The same applies for data from tuning files as well\nas the directly provided data.\n\nData application order:\n\n- profile defaults\n- data from tuning files (in order of appearance) `tuning_files_list`\n- data provided directly (in order of appearance) `tuning_data_list`\n\n\n```python\nimport yacfg\n\n# generating only broker.xml config using default values from profile,\n# no tuning, writing output to a target path\nyacfg.generate(\n    profile=\'artemis/2.5.0/default.yaml.jinja2\',\n    output_filter=[\'broker.xml\'],\n    output_path=\'/opt/artemis-2.5.0-i0/etc/\',\n)\n\n# using both files and direct values, and writing generated configs to\n# a target directory\nyacfg.generate(\n    profile=\'artemis/2.5.0/default.yaml.jinja2\',\n    tuning_files_list=[\n        \'my_values.yaml\',\n        \'machine_specific.yaml\',\n        \'logging_debug.yaml\'\n    ],\n    tuning_data_list=[\n        {\'name\': \'custom name\', \'config\': \'option_a\'},\n        {\'address\': \'10.0.0.1\'},\n        {\'LOG_LEVEL\': \'debug\'},\n    ],\n    output_path=\'/opt/artemis-2.5.0-i0/etc/\',\n)\n\n# just get generated data for further processing, using just tuning files\ndata = yacfg.generate(\n    profile=\'artemis/2.5.0/default.yaml.jinja2\',\n    tuning_files_list=[\n        \'my_values.yaml\',\n        \'machine_specific.yaml\',\n        \'logging_debug.yaml\'\n    ],\n)\nprint(data[\'broker.xml\'])\n```\n\n## Batch configurations\n\nIn case you have multiple services to configure in your environment,\nthat you probably will have at some point, there is a tool for that\nas well. The tool is called yacfg-batch. It has only yaml input and\nit uses yacfg to generate configurations as you are already used to.\n\nInput yaml file defines all services you need to generate, what\nprofiles to use, and what tuning to provide to `yacfg`.\nIt allows you to configure defaults and common for services.\n\n### Batch profile file\n\nAs said it is YAML. It has two special sections: `_default` and `_common`.\nAs the name suggests, `_default` values are used when values are not\ndefined per specific section. Where `_common` is added to the values\nof all sections. The important thing here is that `_default` has lower\npriority than `_common` and that has lower priority than specific section\nvalues.\n\nEvery section has 4 values: `profile`, `template`, `tuning_files`,\n and `tuning`. As the name suggests, `profile` defines what generation profile\n to select, and it directly correlates with `yacfg`\'s `--profile`.\n `template` defines what generation template to use\n (overrides one in the profile if defined), and it directly correlates with\n `--template` from `yacfg`. `tuning_files` option is a list of tuning\n files to use, when combining defaults, commons, and specific values,\n tuning_files list is concatenated. Finally `tuning` is a map of\n specific tuning values, correlates with `--opt` of `yacfg`. When combining\n defaults, commons, and specifics, it will be updated over using python\n dict.update() and it will work only on first level, so it is recommended\n to use flat values for tuning only.\n\nExample:\n```yaml\n\n_default:\n    profile: artemis/2.5.0/default.yaml.jinja2\n    tuning_files:\n      - defaults/broker_default.yaml\n\n_common:\n    tuning_files:\n      - common/security.yaml\n      - common/logging.yaml\n    tuning_values:\n      LOG_LEVEL_ALL: INFO\n\nbrokerA/opt/artemis/etc:\n    pass: true\n\nbrokerB/opt/artemis/etc:\n    profile: artemis/2.5.0/AIOBasic.yaml.jinja2\n    tuning_files:\n      - brokerB/queues.yaml\n\n---\n\n_default:\n    profile: amq_broker/7.2.0/default.yaml.jinja2\n    tuning_files:\n      - defaults/amq_broker_default.yaml\n\nbrokerC/opt/amq/etc:\n    tuning:\n      LOG_LEVEL_ALL: DEBUG\n\n```\n\nAs you can see, `yacfg-batch` supports multiple sections, in single\nbatch profile file, that allows you to generate multiple groups using\nseparated `_default` and `_common` sections for that.\n\n### executing batch\n\nWhen you have defined all tuning files you need, and in the root of this\nbatch configuration you have your batch profile file, you can now simply\nrun `yacfg-batch`:\n\n```bash\n\nyacfg-batch --input [batch_profile_file] --output [output_path]\n```\n\nYou can use multiple input files and all of those will be generated\nconsecutively. In the output path, new subdirectories will be created\nfor every item you configure (every section), section key will be used\nfor that subdirectory. If the section name resembles a path, whole\npath will be created. For example for `brokerA/opt/artemis/etc`\nthe configuration will be generated into\n`[output_path]/brokerA/opt/artemis/etc/`.\n\n## Documentation\nFormatted documentation can be viewed at [rh-messaging-qe.github.io/yacfg/](https://rh-messaging-qe.github.io/yacfg/).\n\n\n## Contributing\n\nIf you find a bug or room for improvement, submit either a ticket or PR.\n\n## Contributors\n\n_Alphabetically ordered_\n\n* Dominik Lenoch <dlenoch@redhat.com> (maintainer)\n* Michal Tóth <mtoth@redhat.com>\n* Otavio Piske <opiske@redhat.com>\n* Sean Davey <sdavey@redhat.com>\n* Zdenek Kraus <zkraus@redhat.com> (author)\n\n## License\n\nCopyright 2018-2021 Red Hat Inc.\n\nLicensed under the Apache License, Version 2.0 (the "License");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n   http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an "AS IS" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.\n\n## Acknowledgments\n\n* [jinja2](http://jinja.pocoo.org/docs/2.10/) -- awesome templating engine\n* [yaml](http://yaml.org/) -- very convenient user readable format\n* [learn_yaml](https://learnxinyminutes.com/docs/yaml/) -- great YAML cheat sheet\n* [pyyaml](https://github.com/yaml/pyyaml) -- python YAML parser\n* [jq](https://stedolan.github.io/jq/) -- great tool for working with structured data (JSON)\n* [yq](https://yq.readthedocs.io/en/latest/) -- YAML variant of jq\n* [github templates examples](https://github.com/stevemao/github-issue-templates/tree/master/simple) -- Nice set of ISSUE_TEMPLATE.md and PULL_REQUESTS_TEMPLATE.md examples\n* [contributing example](https://gist.github.com/PurpleBooth/b24679402957c63ec426) -- example/template of CONTRIBUTING.md\n* [Fedora Project code-of-conduct](https://docs.fedoraproject.org/en-US/project/code-of-conduct/) -- the inspiration for CODE_OF_CONDUCT.md\n',
    'author': 'Zdenek Kraus',
    'author_email': 'zkraus@redhat.com',
    'maintainer': 'Dominik Lenoch',
    'maintainer_email': 'dlenoch@redhat.com',
    'url': 'https://github.com/rh-messaging-qe/yacfg',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import copy
import logging
import os

import yaml

import yacfg.yacfg
from .exceptions import YacfgBatchException

LOG = logging.getLogger(__name__)


class GenerateData(object):
    profile_name = None
    template_name = None
    tuning_files = None
    tuning_data = None

    def __str__(self):
        tmp = (
            'GenerateData({!r}, {!r}, {}, {})'.format(
                self.profile_name,
                self.template_name,
                self.tuning_files,
                self.tuning_data
            )
        )
        return tmp

    __repr__ = __str__

    def __eq__(self, other):
        return (self.profile_name == other.profile_name
                and self.template_name == other.template_name
                and self.tuning_files == other.tuning_files
                and self.tuning_data == other.tuning_data)


def iter_gen_profiles(filename):
    # noinspection PyUnusedLocal
    profile_data_list = None

    try:
        profile_data_list = yaml.load_all(open(filename, 'r'))
    except IOError as exc:
        raise YacfgBatchException(
            'Unable to open gen profile "{}" {}'.format(filename, exc)
        )
    except yaml.YAMLError as exc:
        raise YacfgBatchException(
            'Unable to parse YAML gen profile "{}" {}'.format(filename, exc)
        )

    for profile_data in profile_data_list:
        yield profile_data


def generate(input_files, output_path=None):
    """Main batch generation function, get input files, collects data,
    and uses core yacfg's generate to do the work.

    :param input_files: list of yaml input files with generation
        specification
    :type input_files: list[str]
    :param output_path: path to write generated configurations
    :type output_path: str | None
    """
    for profile_file in input_files:
        LOG.info('- Profile file: {}'.format(profile_file))
        for profile_file_data in iter_gen_profiles(profile_file):
            input_path = os.path.dirname(profile_file)

            default = extract_generate_data(profile_file_data)
            common = extract_generate_data(profile_file_data, '_common')

            generate_all_profiles(
                input_path, output_path, default, common, profile_file_data
            )


def extract_generate_data(profile_file_data, section='_default'):
    """Get all relevant data from special sections like _default

    Data contains profile name, template name, tuning files, and
    tuning data.

    :param profile_file_data: one chunk of batch generation profile
        that is supposed to contain the section
    :type profile_file_data: dict
    :param section: selected section name, default: '_default'
    :type section: str

    :return: Extracted generate data
    :rtype: GenerateData
    """
    section_data = profile_file_data.get(section)
    result = GenerateData()
    if section_data:
        result.profile_name = section_data.get('profile')
        result.template_name = section_data.get('template')
        result.tuning_files = section_data.get('tuning_files')
        result.tuning_data = section_data.get('tuning')
    return result


def prioritize_generate_data(profile, common, default):
    """Gather all available generation data by their priority,
    the highest is the particular profile data, then defaults.
    Common data will be shared even if profile data are available.
    Be aware that with tuning_files and tuning_data, Default has lower
    priority than Common, so if Profile data is not present, Default
    will be applied before common.

    With profile name and template name is priority order:
        profile, common, default
    there is no sharing, since those are single values.

    :param profile: profile relevant generate data
    :type profile: GenerateData
    :param common: common generate data
    :type common: GenerateData
    :param default: default generate data
    :type default: GenerateData

    :return: prioritized data in generate data format
    :rtype: GenerateData
    """
    result = GenerateData()

    for key in ['profile_name', 'template_name']:
        for conf in [default, common, profile]:
            value = getattr(conf, key)
            if value:
                setattr(result, key, value)

    # tuning files
    result.tuning_files = []
    if not profile.tuning_files and default.tuning_files:
        result.tuning_files.extend(default.tuning_files)
    if common.tuning_files:
        result.tuning_files.extend(common.tuning_files)
    if profile.tuning_files:
        result.tuning_files.extend(profile.tuning_files)
    # normalize empty list to None
    if not result.tuning_files:
        result.tuning_files = None

    # tuning data
    result.tuning_data = {}
    if not profile.tuning_data and default.tuning_data:
        result.tuning_data = copy.copy(default.tuning_data)
    if common.tuning_data:
        result.tuning_data.update(common.tuning_data)
    if profile.tuning_data:
        result.tuning_data.update(profile.tuning_data)
    # normalize empty dict to none
    if not result.tuning_data:
        result.tuning_data = None

    return result


def generate_all_profiles(input_path, output_path, default, common,
                          profiles_file_data):
    """Main subroutine for generating all service's profile configs.

    :param input_path: path of used input yaml file, to pick
        dependencies, tuning files, etc.
    :type input_path: str
    :param output_path: path where to generate configs, name of service
        profile will be used as subdirectory
    :type output_path: str
    :param default: collection of default GenerateData
    :type default: GenerateData
    :param common: collection of common GenerateData
    :type common: GenerateData
    :param profiles_file_data: profiles generation data in dict format,
        as loaded from YAML
    :type profiles_file_data: dict
    """
    profile_list = [
        x for x in profiles_file_data.keys() if not x.startswith('_')
    ]

    for profile in profile_list:
        LOG.info('-- Profile: {}'.format(profile))

        profile_data = extract_generate_data(profiles_file_data, profile)
        generate_data = prioritize_generate_data(profile_data, common, default)

        if not generate_data.profile_name:
            raise YacfgBatchException(
                'No selected profile,'
                ' cannot generate_via_tuning_files.'
            )

        # post process tuning files
        if generate_data.tuning_files:
            generate_data.tuning_files = [
                os.path.join(input_path, x) for x in generate_data.tuning_files
            ]

        LOG.debug('tuning data: {}'.format(generate_data.tuning_data))

        target_path = None
        if output_path:
            target_path = os.path.join(output_path, profile)

        tuning_data = None
        if generate_data.tuning_data:
            tuning_data = [generate_data.tuning_data]

        LOG.debug(
            'CALL: yacfg --profile {} --template {} '
            '--tuning {} --output {} # extra tuning: {} '
            '>> {}'.format(
                generate_data.profile_name, generate_data.template_name,
                generate_data.tuning_files, target_path,
                generate_data.tuning_data, target_path
            ))

        yacfg.yacfg.generate(
            profile=generate_data.profile_name,
            template=generate_data.template_name,
            output_path=target_path,
            tuning_files_list=generate_data.tuning_files,
            tuning_data_list=tuning_data,
        )
